;;; wege.scm
;;; stellt die Funktionen fuer die Verwaltung der Wege bereit,
;;; wenn die Knotenliste verwendet wird

(load "gallenbacher-graph-knoten.scm")
; Verwendet wird davon die Knotenliste *knoten*
; und die Zugriffsfunktion (nachfolge-kanten <aktuelle-stadt> <kanten>)

; Wege sind Listen aus der Laenge und den besuchten Knoten (rueckwaerts)
; Muster: (60 X B I)

;;; ===== alle-nachfolge-wege =====
; gibt zum aktuellen Weg alle Wege zurueck, die sich aus den
; Nachfolgekanten der Knotenliste (zB zu C: '((I 40) (M 31) (X 23)) ergeben.
(define
  (alle-nachfolge-wege weg kanten)
  (define
    (alle-intern die-nachfolger)
    (cond
      ((null? die-nachfolger) '())
      ((member (first (first die-nachfolger)) weg) ; nicht zurueck
       (alle-intern (rest die-nachfolger)))
      (else
       (cons
        (neuer-weg weg (first die-nachfolger))
        (alle-intern (rest die-nachfolger))))
      ))
  (alle-intern
   (hole-nachfolger
    (second weg) ; aktuelle-stadt steht nach der Laenge im weg
    kanten)))


;;; ===== Hilfsfunktion neuer-weg =====
; Gibt zu einem Weg und zu einer Kante den verlaengerten Weg zurueck.
(define
  (neuer-weg weg kante)
  (cons
     (+ (second kante) (first weg)) ; neue Weglaenge
     (cons
      (first kante) ; Zielort der Kante vorn einbauen
      (rest weg))))
;Tests:
; (neuer-weg '(34 B I) '(X 26))
; -> '(60 X B I)

;;; ===== alle-nachfolge-wege =====
; Tests:
; (alle-nachfolge-wege '(22 N A) *knoten*)
; -> '((42 D N A) (51 X N A) (52 Z N A))
